RD_API / Resource Description API
=======================================

This is project for Resource Description API for interacting with the description(s) of production resources. 
Specifically these are called Resource Descriptions (RD), and one specific RD is used to thoroughly describe
a production resource, and all its characteristics.
Relates to EU Hz-2020 project ReCaM <http://www.recam-project.eu/>.

Repository location
--------------------
The main storage for this GIT repository can be found from:
<https://bitbucket.org/nikkeSi/rd_api>

Administrator of this GIT repository is:  
Tampere University of Technology (TUT) /  
Laboratory of Mechanical Engineering and Industrial Systems (MEI) /  
Niko Siltala  
niko.siltala@tut.fi  
+358-40-536 6017  

Latest compiled packages can be found from the Maven repository:
<http://resourcedescription.tut.fi/maven/fi/tut/mei/resdescapi/RD_API/>

Authors
-------
The main authors for this repository are:

* Niko Siltala, niko.siltala@tut.fi




Prerequisities
---------------

### Softwares to be installed:

Software Versions used for the application. If specific folder location is required, it is listed here. Any other additional information that applies.

Software | Version | to/path | Additional information
-------- | ------- | ------- | ----------------------
OpenJDK 64-Bit	| 1.8.0	| 	
Oracle Java JDK	| 1.8 | |
Maven		| 3.0.5
JAXB    |       |     | |


### Other requirements:
#### SW1

Compilation:
------------
Application will be compiled, deployed, etc. with Maven.  
Make sure Maven version is correct and that Maven uses right java version.
```shell
mvn -v		-> Checks which version of Maven is in use
java -version	-> Java version used
```

Application can be compiled and executed with:
```shell
mvn compile			-> Compiles project
mvn test			-> Compile and run Unit tests
mvn install			-> Installs RD API project to local Maven repository for use
mvn compile assembly:single	-> Compiles and makes jar package, which includes all dependency jar packages.
mvn exec:java			-> executes the RD API console application
mvn exec:java -Dexec.args="-pi"			-> executes the RD API console application with arguments

```

Other Maven targets and actions:
```shell
mvn jaxb2:xjc   		-> Makes JAXB binding and generates java classes from XSDs
mvn compile			-> Compiles project
mvn test			-> Compile and run Unit tests
mvn package			-> Compiles, tests and makes jar package
mvn compile assembly:single	-> Compiles and makes jar package, which includes all dependency jar packages.
mvn exec:java			-> executes the RD API console application
mvn javadoc:javadoc		-> java doc generation
mvn install			-> Compiles, tests and makes jar package which is then copied to local maven repository
mvn clean			-> Cleans project folders from compiled files
```

The pom.xml file should be modified if the resources are located some other directory than given in Prerequisities.


EXECUTING and Testing the API:
------------------------------
Properties file with name of `resourceDescription.properties` must be passed to the API. It can be directly located to the same folder where the API library is executed or its location can be passed as system property like in following example `-DresourceDescription.properties=./src/main/resources/resourceDescription.properties`. The semantics of the properties file is XML following the XML Schema `ResDescApiProperties.xsd`, which can be found inside the jar file `/resources/schemas/ResDescApiProperties.xsd`.

A full API call could look like this:
```shell
java -DresourceDescription.properties=./src/main/resources/resourceDescription.properties -jar ./RD_API-0.2.0-DEV-jar-with-dependencies.jar -tc
```

FOLDER STRUCTURE:
-----------------

The folder structure follows standard Maven folder structures.

Folder | Description 
--- | ---
doc/				| Documentation files. Mainly the java code documentation.
lib/				| Library files. Third party components. (obsolete. Maven takes care of these)
src/main/java/			| Java source codes
src/main/resources/		| Resource files
src/main/resources/img/		| Image files for application
src/main/resources/rd/		| Sample resource description (RD) files
src/main/resources/schemas/		| XML Schema files
src/main/xjb/		| JAXB binding instructions
src/test/java/			| Test files
src/test/resources/			| Additional resources files for tests
target/				| compiled targets
target/generated-sources/jaxb/		| Generated java sources through JAXB binding.
resources/	| resource files for the application. TBD how differes from src/main/resources/rd/
misc/			| Miscellaneous files
pom.xml				| Maven targets


# USING the API #
Some usage instructions for the Resource Description API (RD_API).

## Pre-requisities ##
1. Create and modify the file `resourceDescription.properties` providing configurations for the API application. The XML Schema (XSD) can be found from inside the jar file from the following location `/resources/schemas/ResDescApiProperties.xsd`.
2. Make sure the application execution founds the properties file. One can use the command line App inside API to test this.

## Operating with the API - Initialisation ##
1. Create and access Resource Catalogue Storage Pool. Create instance of fi.tut.mei.resdescapi.rescatapi.ResCatStoPool. This class uses the configuration for intialising the storage implementations. One can use methods getStorageKeys() and getStorage_byName(String name) to get an instance of specific ResCatStoImpl(ementation).
2. Create instance(s) of class implementing abstract class fi.tut.mei.resdescapi.rescatapi.ResCatAPI. Some existing ones are fi.tut.mei.resdescapi.rescatapi.ResCatAPI_ById; ResCatAPI_ByName; ResCatAPI_ByDescription; ResCatAPI_ByGuid. Each of these classes can be used for querying information associated to specific field. Argument of method, such as viewARD_Description(String ardDesc), is used for different purpose and search field depending, which class is instantiated. So e.g. in case of ResCatAPI_ById ardDesc data is search among the Id field data.  
3. After instantiation use method setResCatStoHandler(ResCatStorageInterface resCatStoHandler) to define which storage implementation (from Pool) should handle these requests.
4. One can also alter the search type of ResCatAPI by using method setSearchType(EnumSearchType searchType). The default search type is EXACT, so exact matches are looked after. Other options can set at any time before executing the new search.

## Operating with the API - Making Searches & Requests ##
1. Use instance of fi.tut.mei.resdescapi.rescatapi.ResCatAPI, created at previous steps, for your searches. Available search methods are defined in fi.tut.mei.resdescapi.rescatapi.ResCatAPIInterface. See method details from there.



# README template #

This README would normally document whatever steps are necessary to get your application up and running.

### What is this repository for? ###

* Quick summary
* Version
* [Learn Markdown](https://bitbucket.org/tutorials/markdowndemo)

### How do I get set up? ###

* Summary of set up
* Configuration
* Dependencies
* Database configuration
* How to run tests
* Deployment instructions

### Contribution guidelines ###

* Writing tests
* Code review
* Other guidelines

### Who do I talk to? ###

* Repo owner or admin
* Other community or team contact
